/*
 * Mirai. Copyright (C) 2024 Vyn
 * This file is licensed under version 3 of the GNU General Public License (GPL-3.0-only)
 * The license can be found in the LICENSE file or at https://www.gnu.org/licenses/gpl-3.0.txt
 */

#pragma once
#include "Mirai.h"
#include "TaskItem.h"
#include "using.h"
#include <cstddef>
#include <string>
#include <vector>

namespace mirai
{

struct FilteredEvent {
	Event *event;
	std::vector<TaskItem *> filteredTasks;
};

struct FilteredDay {
	Day *day;
	std::vector<FilteredEvent> filteredEvents;
	std::vector<TaskItem *> filteredTasks;
};

class TasksView
{

  public:
	TasksView(Mirai *mirai);

	FilteredDay &operator[](int index);
	std::vector<TaskItem *> &filteredUnscheduledTasks();

	size_t count() const;
	void update();
	void addTagFilter(const std::string &tag);
	void removeTagFilter(const std::string &tag);
	void addSourceFilter(const std::string &fileName);
	void removeSourceFilter(const std::string &fileName);
	void removeFilters();
	const Tags &getActiveTagsFilter();
	const std::vector<std::string> &getActiveFilesFilter();
	bool isSourceFilterActive(const std::string &sourceName);

	void hideCompletedTasks(bool hide);
	bool shouldHideCompletedTasks() const;

  private:
	std::vector<FilteredDay> filteredDays;
	std::vector<TaskItem *> filteredUnscheduledTasks_;

	Mirai *mirai;
	Tags tagsFilter;
	std::vector<std::string> resourcesFilter;
	bool shouldHideCompletedTasks_ = true;
};
} // namespace mirai
